#!/usr/local/bin/perl -w

=head1 NAME

p4.t - testing of vcp p4 i/o

=cut

use strict ;

use Carp ;
use File::Spec ;
use POSIX ':sys_wait_h' ;
use Test ;
use VCP::TestUtils ;

my @vcp = ( vcp_cmd, "vcp:-" );

my $t = -d 't' ? 't/' : '' ;

# what change number to start incremental export at
my $first_import_1_change;

## Repositories to read from
my $p4root_0 = "${t}p4root_0";
my $p4root_1 = "${t}p4root_1";

## where to put the destination repository
my $cvsroot = tmpdir "cvsroot";

## where to write to in the destination repo
my $cvs_module = "p4_t_module";

my $cvs_spec = "cvs:$cvsroot:$cvs_module/";


my @tests = (
##
## p4->cvs->revml bootstrap
##
sub {
   eval {
      run \@vcp, \<<VCP_FILE;
Source:      p4:$p4root_0://depot/foo/... --run-p4d

Destination: $cvs_spec --init-cvsroot --delete-cvsroot

Map:
## ASSumes directories under //depot/foo/ are the main and branch
## dirs.
        */(...)<(*)>  \$1<\$2>
VCP_FILE
   };

   ok $@ || '', '';
},

sub {
   return skip "previous test failed", 1 if $@;

   eval{
      my $infile  = $t . "test-cvs-in-0.revml" ;
      my $in = slurp $infile ;
      my $out = get_vcp_output $cvs_spec, qw( -r 1.1: ) ;

      s_content  qw( rep_desc time user_id ),   \$in, \$out ;
      s_content  qw( rev_root ),                \$in, $cvs_module ;
      rm_elts    qw( cvs_info change_id mod_time ), \$in ;
      rm_elts    qw( label ), qr/vcp_.*/, \$out ;
#
#      $out =~ s{<rev_id>1.}{<rev_id>}g ;
#      $out =~ s{<base_rev_id>1.}{<base_rev_id>}g ;
#      $out =~ s{((id="|_id>)?[^>]*#)1\.}{$1}g;
#
      $in =~ s{(id="|_id>)/+ignored}{$1/$cvs_module}g;
#
#      $in =~ s{(<rev id=.*)main-branch-1/branched#(\d+)}{\1main/branched#1.2.

      assert_eq $infile, $in, $out ;
   } ;
   ok (( $@ || '' ), '', 'diff' );
},

sub { exit },

# determine next change number that p4d will use this will become the
# change number of the first change in t/test-p4-in-1.revml when it is
# imported.
sub {
   my $p4_options = launch_p4d "p4_", { repo_dir => $p4root_0 };

   # get counter change
   run_p4 [ qw( counter change ) ],
      \undef,
      \$first_import_1_change,
      $p4_options ;

   chomp $first_import_1_change ;
   die "Invalid change counter value: '$first_import_1_change'"
      unless $first_import_1_change =~ /^\d+$/ ;
   ++$first_import_1_change ;

   $p4_options->{p4d_handle}->kill_kill;

   ok 1;
},



##
## p4->cvs->revml incremental export
##
sub { ## Two ok's in next test.
},

sub {
   eval {
      run \@vcp, \<<VCP_END;
Source:       p4:$p4root_1://depot/...\@$first_import_1_change,#head --run-p4d

Destination:  $cvs_spec
VCP_END

      ok 1 ;

      my $infile  = $t . "test-p4-in-1.revml" ;
      my $in = slurp $infile ;
      my $out = get_vcp_output $cvs_spec, "-r", "ch_$first_import_1_change:" ;

      s_content  qw( rep_desc rep_type time user_id ), \$in, \$out ;
      s_content  qw( rev_root ),                       \$in, $cvs_module ;
      rm_elts    qw( p4_info change_id ),              \$in ;
      rm_elts    qw( label ), qr/vcp_.*/,              \$in, \$out ;

      $out =~ s{<rev_id>1.}{<rev_id>}g ;
      $out =~ s{<base_rev_id>1.}{<base_rev_id>}g ;
      $out =~ s{((id="|_id>)?[^>]*#)1\.}{$1}g;

      $in =~ s{(id="|_id>)/+ignored}{$1/$cvs_module}g;

      rm_elts    qw( branches ),                      \$in ;

      ## TODO: This is an incremental export and perforce does not give us
      ## a trivial way to figure out what branch a file is on (since
      ## multiple branch views can cover the same file), so we bail
      ## on that for now.
      rm_elts    qw( branch_id ),                     \$in ;

      assert_eq $infile, $in, $out ;
   } ;
   ok $@ || '', '', 'diff' ;
},

);  # end @tests.



plan tests => scalar @tests, todo => [ 4 ] ;

my $p4d_borken = $ENV{P4BORKEN}  || p4d_borken ;
my $cvs_borken = $ENV{CVSBORKEN} || cvs_borken;


my $why_skip ;
$why_skip .= "p4 command not found\n"  unless ( `p4 -V`  || 0 ) =~ /^Perforce/ ;
$why_skip .= "$p4d_borken\n"           if $p4d_borken ;
$why_skip .= "$cvs_borken\n"           if $cvs_borken ;

$why_skip ? skip( $why_skip, '' ) : $_->() for @tests ;
